diff --git a/src/Fl_Native_File_Chooser_GTK.cxx b/src/Fl_Native_File_Chooser_GTK.cxx
index 2d6b819f9..9fcce874e 100644
--- a/src/Fl_Native_File_Chooser_GTK.cxx
+++ b/src/Fl_Native_File_Chooser_GTK.cxx
@@ -124,7 +124,11 @@ private:
   Fl_GTK_Native_File_Chooser_Driver(int val);
   virtual ~Fl_GTK_Native_File_Chooser_Driver();
   static int did_find_GTK_libs;
+  static int instance;
+  static void *ptr_glib;
+  static void *ptr_gtk;
   static void probe_for_GTK_libs(void);
+  static void free_GTK_libs(void);
   virtual void type(int);
   virtual int count() const;
   virtual const char *filename() const;
@@ -139,8 +143,6 @@ private:
 };
 
 
-int Fl_GTK_Native_File_Chooser_Driver::did_find_GTK_libs = 0;
-
 /* These are the GTK/GLib methods we want to load, but not call by name...! */
 
 // void g_free (gpointer mem);
@@ -292,6 +294,10 @@ static XX_gtk_toggle_button_set_active fl_gtk_toggle_button_set_active = NULL;
 
 
 int Fl_GTK_Native_File_Chooser_Driver::have_looked_for_GTK_libs = 0;
+int Fl_GTK_Native_File_Chooser_Driver::did_find_GTK_libs = 0;
+int Fl_GTK_Native_File_Chooser_Driver::instance = 0;
+void *Fl_GTK_Native_File_Chooser_Driver::ptr_glib = NULL;
+void *Fl_GTK_Native_File_Chooser_Driver::ptr_gtk = NULL;
 
 
 Fl_Native_File_Chooser::Fl_Native_File_Chooser(int val) {
@@ -319,6 +325,8 @@ Fl_GTK_Native_File_Chooser_Driver::Fl_GTK_Native_File_Chooser_Driver(int val) :
   gtkw_title = NULL;    // dialog title
   _btype = val;
   previous_filter = NULL;
+  instance++;
+  //printf("instance #%d\n", instance);
 }
 
 Fl_GTK_Native_File_Chooser_Driver::~Fl_GTK_Native_File_Chooser_Driver()
@@ -343,6 +351,11 @@ Fl_GTK_Native_File_Chooser_Driver::~Fl_GTK_Native_File_Chooser_Driver()
   }
   gtkw_count = 0; // assume we have no files selected now
   gtkw_title = strfree(gtkw_title);
+  instance--;
+  if (instance < 1) {
+    //printf("free GTK libs\n");
+    free_GTK_libs();
+  }
 }
 
 void Fl_GTK_Native_File_Chooser_Driver::type(int val) {
@@ -698,7 +711,7 @@ dlerror();    /* Clear any existing error */  \
 fl_##SSS = (XX_##SSS)dlsym(LLL, #SSS);        \
 if ((pc_dl_error = dlerror()) != NULL)  {     \
 fprintf(stderr, "%s\n", pc_dl_error);       \
-did_find_GTK_libs = 0;                      \
+free_GTK_libs();                           \
 return; }
 
 static void* fl_dlopen(const char *filename1, const char *filename2)
@@ -716,9 +729,8 @@ static void* fl_dlopen(const char *filename1, const char *filename2)
  */
 void Fl_GTK_Native_File_Chooser_Driver::probe_for_GTK_libs(void) {
 #if HAVE_DLSYM && HAVE_DLFCN_H
-  void *ptr_glib    = NULL;
-  void *ptr_gtk     = NULL;
-  
+  if (did_find_GTK_libs) return;
+
 #   ifdef __APPLE_CC__ // allows testing on Darwin + X11
   ptr_glib    = dlopen("/sw/lib/libglib-2.0.dylib", RTLD_LAZY | RTLD_GLOBAL);
 #   else
@@ -748,7 +760,7 @@ void Fl_GTK_Native_File_Chooser_Driver::probe_for_GTK_libs(void) {
 #ifdef DEBUG
     puts("Failure to load libglib or libgtk");
 #endif
-    did_find_GTK_libs = 0;
+    free_GTK_libs();
     return;
   }
   
@@ -796,6 +808,21 @@ void Fl_GTK_Native_File_Chooser_Driver::probe_for_GTK_libs(void) {
 #endif // HAVE_DLSYM && HAVE_DLFCN_H
 } // probe_for_GTK_libs
 
+void Fl_GTK_Native_File_Chooser_Driver::free_GTK_libs(void)
+{
+#if HAVE_DLSYM && HAVE_DLFCN_H
+  if (ptr_gtk) {
+    dlclose(ptr_gtk);
+    ptr_gtk = NULL;
+  }
+  if (ptr_glib) {
+    dlclose(ptr_glib);
+    ptr_glib = NULL;
+  }
+  did_find_GTK_libs = 0;
+#endif // HAVE_DLSYM && HAVE_DLFCN_H
+}
+
 #endif // FL_CFG_WIN_X11
 
 //
